﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using Infragistics.Controls.Charts;
using Infragistics.Windows;
using Infragistics.Controls;
using System.Diagnostics;

namespace DataChartZooming
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        bool fromMouseWheel = false;
        double scaleH = -1;
        double scaleV = -1;
        double positionH = -1;
        double positionV = -1;

        public MainWindow()
        {
            InitializeComponent();


            DataContext = Data.GetData(15, 20, 50);

            xmLineChart.WindowPositionHorizontal = .8;
            xmLineChart.WindowScaleHorizontal = .2;


        }

        void xmLineChart_WindowRectChanged(object sender, Infragistics.RectChangedEventArgs e)
        {
            var xAxis = xmLineChart.Axes.OfType<CategoryXAxis>().First();
            var yAxis = xmLineChart.Axes.OfType<NumericYAxis>().First();

            var viewport = new Rect(0, 0, xAxis.ActualWidth, yAxis.ActualHeight);
            var window = xmLineChart.WindowRect;

            var unscaledXStart = xAxis.GetUnscaledValue(0, window, viewport);
            var unscaledXEnd = xAxis.GetUnscaledValue(xAxis.ActualWidth, window, viewport);

            if (!double.IsNaN(unscaledXStart) && !double.IsNaN(unscaledXEnd))
            {
                int startIndex = (int)unscaledXStart;
                int endIndex = (int)unscaledXEnd;

                if (startIndex > 0)
                    startIndex--;

                if (endIndex < (DataContext as ObservableCollection<Data>).Count - 1)
                    endIndex++;

                List<int> rangeItems = new List<int>();
                for (int i = startIndex; i < endIndex; i++)
                {
                    rangeItems.Add((DataContext as ObservableCollection<Data>)[i].Value);
                }

                yAxis.MinimumValue = rangeItems.Min();
                yAxis.MaximumValue = rangeItems.Max();
            }

            if (fromMouseWheel)
            {
                (sender as XamDataChart).WindowPositionHorizontal = positionH;
                (sender as XamDataChart).WindowPositionVertical = positionV;
                (sender as XamDataChart).WindowScaleHorizontal = scaleH;
                (sender as XamDataChart).WindowScaleVertical = scaleV;
                fromMouseWheel = false;
            }
        }

        private void xmLineChart_PreviewMouseWheel(object sender, MouseWheelEventArgs e)
        {
            fromMouseWheel = true;
            scaleH = xmLineChart.WindowScaleHorizontal;
            scaleV = xmLineChart.WindowScaleVertical;
            positionH = xmLineChart.WindowPositionHorizontal;
            positionV = xmLineChart.WindowPositionVertical;
        }

    }
    
    public class Data : INotifyPropertyChanged
    {
        private int val;
        private string label;

        public int Value
        {
            get { return val; }
            set 
            { 
                val = value;
                PropChanged("Value");
            }
        }

      
        public string Label
        {
            get { return label; }
            set
            {
                label = value;
                PropChanged("Label");
            }
        }


        public static ObservableCollection<Data> GetData(int points, int rangeStart, int rangeEnd)
        {
            ObservableCollection<Data> data = new ObservableCollection<Data>();
            Random r = new Random();
            for (int i = 0; i < points; i++)
            {
                data.Add(new Data { 
                    Label = "Item " + i,
                    Value = r.Next(rangeStart, rangeEnd),
                });
            }

            return data;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public void PropChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }
}
